#! /bin/sh
#
# Copyright by The HDF Group.
# All rights reserved.
#
# function definistions used by Nagg test scripts.

TESTNAME=nagg
EXIT_SUCCESS=0
EXIT_FAILURE=1

NAGG=nagg               	   # The tool name
NAGG_BIN=`pwd`/../src/$NAGG    # The path of the NAGG binary

CMP='cmp'
DIFF='diff -c'
NLINES=20		    # Max. lines of output to display if test fails
RM="rm -f"


TESTFILE=tmptestfile.h5	    # name of temporary HDF5 testfile
nerrors=0
verbose=yes
h5haveexitcode=yes	    # default is yes

# source dirs
SRC_TOOL="$srcdir"
SRC_TOOL_TESTFILES="$SRC_TOOL/testfiles"
# The build (current) directory need to have a testfiles directory.
# Not used yet.
BUILD_TOOL=.
BUILD_TOOL_TESTFILES=$BUILD_TOOL/testfiles
# create local test directory if it does not exist.
test -d $BUILD_TOOL_TESTFILES

# RUNSERIAL is used. Check if it can return exit code from executable correctly.
if [ -n "$RUNSERIAL_NOEXITCODE" ]; then
    echo "***Warning*** Serial Exit Code is not passed back to shell correctly."
    echo "***Warning*** Exit code checking is skipped."
    h5haveexitcode=no
fi

# Print a $* message left justified in a field of 70 characters
#
MESSAGE() {
   SPACES="                                                               "
   echo "$* $SPACES" | cut -c1-70 | tr -d '\012'
}


# Display the actual stdout ($1), actual stderr ($2) and the optional
# expected output ($3).
DISPLAY_OUTPUT(){
    if [ $# -lt 2 -o $# -gt 3 ]; then
	echo "DISPLAY_OUTPUT: wrong arguments ($*)"
    fi
    echo "**** stdout ****"
    cat $1
    echo "**** stderr ****"
    cat $2
    if [ $# -eq 3 ]; then
	echo "**** expected output ****"
	cat $3
    fi
    echo "****  DONE  ****"
}



# Since the nagg output often displays filenames that contain time dependent
# Creation time, we employ the sed command to change the Creation time into
# generic format so that the output would match.
# First files are input files
# Last file is the result file
TRANSFORM()
{
    # must have at least 1 input file and 1 result file
    if [ $# -lt 2 ]; then
	echo "TRANSFORM: need at least 2 arguments. Only ($*) given."
	return 1
    fi
    infiles=$1
    shift
    while [ $# -gt 1 ]; do
	infiles="$infiles $1"
	shift
    done
    outfile=$1

    # The creation time pattern is: "npp_d[0-9]*_t[0-9]*_e[0-9]*_b[0-9]*_c[0-9]*_".
    # Replace "_c[0-9]*_" with "_cYYYYMMDDHHMMSSssssss_".  Use g because there could
    # be multiple appearances of the creation time pattern in a line.
    sed -e '/npp_d[0-9]*_t[0-9]*_e[0-9]*_b[0-9]*_c[0-9]*_/s/_c[0-9]*_/_cYYYYMMDDHHMMSSssssss_/g' $infiles > $outfile
}


# Grab the output file names from the nagg output
# $1: the output file from Nagg
GET_OUTPUT_FNAME()
{
    output_filenames=`cat $1 | sed -e '/^Produced .*npp_d.*_c.*\.h5$/s/^Produced.*granules in //'`
}


# Run tool test.
# It first verifies that the expected code matches the actual exit code.
# If they do not match, the test has failed.
# If they match and if Expected output is not "null", it verifies that
# the Expected output matches the actual output.
# If they do not match, the test has failed.
# Otherwise, the test has passed.
#
# Extension: if the optional "dump" option is given, after the intial
# test matching as described above, the TOOLTEST will dump (--debug)
# the output files produced and match the output with the expected output.
#
# Matching output:
# Since the nagg output often displays filenames that contain time dependent
# creation time, we employ the sed command to change the Creation time into
# generic format so that the output would match.
#
# Matching the dump output:
# TOOLTEST will first scan the file name of the produced output files;
# then run it again with --debug to match if the dump output matches the
# expected output.
#
# It does not show the actual stdout and stderr results if test has passed.
# It displays them otherwise.
# When $NAGG_DEBUG is defined, stdout and stderr are always shown.
#
# Generated output files are removed unless $NAGG_NOCLEANUP is defined.
# $1: Expected exit code
# $2: Expected output ("null" means no expected output)
# $3  Optional dump option (" dump  if given)
# the rest: auguments to call the tool
#
# Note: adding pid ($$) to temporary files so that this code may run multiple
# instances in parallel.
TOOLTEST() 
{
    actual_stdout=tmp-out.$$
    actual_stderr=tmp-err.$$
    actual_output=tmp-output.$$
    # Expected exit code
    expect_code=$1
    shift
    # Expected output
    expected_output=$1
    shift
    if [ $expected_output == "null" ]; then
	# reset to nothing
	expected_output=
    else
	# reset to srcdir
	expected_output=${SRC_TOOL_TESTFILES}/$expected_output
    fi
    # Is the optional "dump" option given?
    if [ "$1" == "dump" ]; then
	dump_option=yes
	shift
    else
	dump_option=	# not defined
    fi

    MESSAGE Testing $@

    $NAGG_BIN $@ > $actual_stdout 2> $actual_stderr
    actual_code=$?

    # save all the output file names produced by nagg.
    GET_OUTPUT_FNAME $actual_stdout

    # verify the exit code.
    if [ $expect_code -eq $actual_code ]; then
	if [ -z "$expected_output" ]; then
	    echo " PASSED"
	    if [ -n "$NAGG_DEBUG" ]; then
		DISPLAY_OUTPUT $actual_stdout $actual_stderr
	    fi
	else
	    # if dump_option is on, find out the -t option.
	    if [ -n "$dump_option" ]; then
		xoptions=
		while [ $# -gt 1 ]; do
		    if [ $1 == -t ]; then
			xoptions="$1 $2"
			break
		    fi
		    shift
		done
		# dump all the granules from the produced files, append to existing
		# stdout and stderr
		$NAGG_BIN --debug $xoptions $output_filenames >> $actual_stdout 2>> $actual_stderr
	    fi
	    TRANSFORM $actual_stdout $actual_stderr $actual_output
	    if $CMP $actual_output $expected_output > /dev/null 2>&1 ; then
		echo " PASSED"
		if [ -n "$NAGG_DEBUG" ]; then
		    DISPLAY_OUTPUT $actual_stdout $actual_stderr $expected_output
		fi
	    else
		nerrors=`expr $nerrors + 1`
		echo "*FAILED*"
		$DIFF $actual_output $expected_output
		DISPLAY_OUTPUT $actual_stdout $actual_stderr $expected_output
	    fi
	fi
    else
	nerrors=`expr $nerrors + 1`
	echo "*FAILED*"
	DISPLAY_OUTPUT $actual_stdout $actual_stderr
    fi

    # Clean up output files
    if test -z "$NAGG_NOCLEANUP"; then
	$RM $actual_stdout $actual_stderr *.h5
    fi
}

# run the tool test and expect succeed (exit code 0) without expected output
CHECK()
{
    TOOLTEST 0 null $*
}


# run the tool test and expect succeed (exit code 0) with expected output
CHECK_OUTPUT()
{
    TOOLTEST 0 $*
}


# run the tool test and expect failure (exit code 1) without expected output
CHECK_FAIL()
{
    TOOLTEST 1 null $*
}

# run the tool test and expect failure (exit code 1) with expected output
CHECK_FAIL_OUTPUT()
{
    TOOLTEST 1 $*
}

# Print the test and indicate it is skipped.
SKIP()
{
    MESSAGE  $*
    echo " -SKIP-"
}

#do spot checks that nagg --version outputs some of the strings that should aways be present:  "nagg: Version "; "HDF5_HL_REGION"..."using external libraries:"; "HDF5 library version:"..."using external libraries:" 
#check that the Major, Minor, and Release version numbers in the output match the numbers stored in nagg.h
CHECK_VERSION()
{
MESSAGE "Testing --version output"
ver_str1=`../src/nagg --version 2>&1 | grep "nagg: Version " | cut -d ' ' -f 3 | sed -e 's/,//'  | cut -d- -f1`
naggversmajor=`grep '#define NAGG_VER_MAJOR' $srcdir/../src/nagg.h | awk '{print $NF}'`
naggversminor=`grep '#define NAGG_VER_MINOR' $srcdir/../src/nagg.h | awk '{print $NF}'`
naggversrelease=`grep '#define NAGG_VER_RELEASE' $srcdir/../src/nagg.h | awk '{print $NF}'`

if test -z "$ver_str1"; then
    echo "Version string not found as expected in --version output."
    echo "                                                                       FAILED"
    nerrors=`expr $nerrors + 1`
#    exit 1
else

    ver_str2=$naggversmajor.$naggversminor.$naggversrelease

    #echo $ver_str1
    #echo $ver_str2

    if [ "${ver_str1}" != "${ver_str2}" ]; then
        echo "$ver_str1 is different from $ver_str2"
        echo "                                                                       FAILED"
        nerrors=`expr $nerrors + 1`
        #    exit 1
    else

        ver_str3=`../src/nagg --version 2>&1 | grep "HDF5_HL_REGION" | grep "using external libraries:"`

        if test -z "$ver_str3"; then
            echo "HDF5_HL_REGION version string not found as expected in --version output."
            echo "                                                                       FAILED"
            nerrors=`expr $nerrors + 1`
            #    exit 1
        else

            ver_str3=`../src/nagg --version 2>&1 | grep "HDF5 library version:" | grep "using external libraries:"`

            if test -z "$ver_str3"; then
                echo "HDF5 library version string not found as expected in --version output."
                echo "                                                                       FAILED"
                nerrors=`expr $nerrors + 1`
                #    exit 1
            else
                echo "                                                                       PASSED"
            fi
        fi
    fi
fi
}
